-- Test of Formlets in a Gitit plugin.
-- Derived from http://www.haskell.org/haskellwiki/Formlets
--
-- Plugin is activated by a code block of the following format:
--
-- ~~~ {.formlet}
--
-- ~~~

-- The plugin presents an input form with four fields for user name,
-- password, month number and day of the month. Upon the POST submission,
-- whose action is this page again, the request is parsed for possible
-- input fields and validated for correctness of month and day.
-- An error message (in red) or success message (in blue) is presented
-- depending on the validation results.
-- Upon the first activation of the page (usually via GET request) with
-- no input fields incluses with the request, a message is shown that
-- no information was entered (the Formlet validation procedure
-- would report error in such case).

module Formlet (plugin) where

import Control.Applicative
import Control.Applicative.Error
import Control.Applicative.State
import Network.Gitit.Interface hiding (User)
import Text.XHtml.Strict.Formlets
import Text.XHtml.Strict ((+++),(<<),(!))
import qualified Text.XHtml.Strict as X
import Happstack.Server.HTTP.Types
import Data.ByteString.Lazy.UTF8 (toString)

plugin :: Plugin

plugin = mkPageTransformM transformBlock

-- Definitions from the Haskell Wiki example.

type MyForm a = XHtmlForm IO a

data User = User {name :: String, pass :: String, birthdate :: Date}
  deriving Show

data Date = Date {month :: Integer, day :: Integer}
  deriving Show
 
dateComponent :: MyForm Date 
dateComponent = Date <$> inputInteger (Just 1) <*> inputInteger (Just 16)

validDate :: Date -> Bool
validDate (Date m d) = m `elem` [1..12] && d `elem` [1..31]
 
dateFull :: MyForm Date
dateFull = dateComponent `check` ensure validDate "This is not a valid date"
 
userFull :: MyForm User
userFull = User <$> input Nothing <*> password Nothing <*> dateFull

-- Create and validate a form. This function differs slightly from
-- the original example to save some code from repetition.

createForm :: Env -> MyForm a -> PluginM (Failing a, X.Html)

createForm env frm = do
  let (extractor, xml, endState) = runFormState env frm
  xml' <- liftIO xml
  ext' <- liftIO extractor
  return (ext', X.form ! [X.method "POST"] << (xml' +++ X.submit "submit" "Submit"))

-- The plugin transformer. If a suitable code block is detected on the page,
-- retrieve the input fields information from the request, convert it into a
-- formlet's environment, and (re) create the form with validation. Form a message
-- based on the validation results.

transformBlock (CodeBlock (_, classes, namevals) contents) | "formlet" `elem` classes = do
  req <- askRequest
  let inps = rqInputs req
      vals = zip (map fst inps) (map (Left . toString . inputValue . snd) inps)
  (ext, frm) <- createForm vals userFull
  let mkhtml = RawHtml . X.showHtmlFragment 
      mkmsg c t = mkhtml $ X.thediv ! [X.thestyle $ "color: " ++ c] << X.primHtml t
      valmsg = case ext of
        Failure _ | null vals -> mkmsg "gray" "No data was previously entered"
        Success a -> mkmsg "blue" $ "Success: " ++ show a
        Failure s -> mkmsg "red" $ "Error " ++ concat s
  doNotCache
  return $ BlockQuote [valmsg
                      ,RawHtml $ X.showHtmlFragment frm]

transformBlock x = return x

